<?php

namespace Ionut\Biblio\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Ionut\Biblio\Http\Requests\AgendaRequest;
use Ionut\Biblio\Interfaces\AgendaRepositoryInterface;
use Ionut\Biblio\Models\Agenda;

/**
 * Resource controller class for agenda.
 */
class AgendaResourceController extends BaseController
{

    /**
     * Initialize agenda resource controller.
     *
     * @param type AgendaRepositoryInterface $agenda
     *
     * @return null
     */
    public function __construct(AgendaRepositoryInterface $agenda)
    {
        parent::__construct();
        $this->repository = $agenda;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Ionut\Biblio\Repositories\Criteria\AgendaResourceCriteria::class);
    }

    /**
     * Display a list of agenda.
     *
     * @return Response
     */
    public function index(AgendaRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Ionut\Biblio\Repositories\Presenter\AgendaPresenter::class)
                ->$function();
        }

        $agendas = $this->repository->paginate();

        return $this->response->title(trans('biblio::agenda.names'))
            ->view('biblio::agenda.index', true)
            ->data(compact('agendas'))
            ->output();
    }

    /**
     * Display agenda.
     *
     * @param Request $request
     * @param Model   $agenda
     *
     * @return Response
     */
    public function show(AgendaRequest $request, Agenda $agenda)
    {

        if ($agenda->exists) {
            $view = 'biblio::agenda.show';
        } else {
            $view = 'biblio::agenda.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('biblio::agenda.name'))
            ->data(compact('agenda'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new agenda.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AgendaRequest $request)
    {

        $agenda = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('biblio::agenda.name')) 
            ->view('biblio::agenda.create', true) 
            ->data(compact('agenda'))
            ->output();
    }

    /**
     * Create new agenda.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AgendaRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $agenda                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('biblio::agenda.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('biblio/agenda/' . $agenda->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/biblio/agenda'))
                ->redirect();
        }

    }

    /**
     * Show agenda for editing.
     *
     * @param Request $request
     * @param Model   $agenda
     *
     * @return Response
     */
    public function edit(AgendaRequest $request, Agenda $agenda)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('biblio::agenda.name'))
            ->view('biblio::agenda.edit', true)
            ->data(compact('agenda'))
            ->output();
    }

    /**
     * Update the agenda.
     *
     * @param Request $request
     * @param Model   $agenda
     *
     * @return Response
     */
    public function update(AgendaRequest $request, Agenda $agenda)
    {
        try {
            $attributes = $request->all();

            $agenda->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('biblio::agenda.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('biblio/agenda/' . $agenda->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('biblio/agenda/' . $agenda->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the agenda.
     *
     * @param Model   $agenda
     *
     * @return Response
     */
    public function destroy(AgendaRequest $request, Agenda $agenda)
    {
        try {

            $agenda->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('biblio::agenda.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('biblio/agenda/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('biblio/agenda/' . $agenda->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple agenda.
     *
     * @param Model   $agenda
     *
     * @return Response
     */
    public function delete(AgendaRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('biblio::agenda.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('biblio/agenda'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/biblio/agenda'))
                ->redirect();
        }

    }

    /**
     * Restore deleted agendas.
     *
     * @param Model   $agenda
     *
     * @return Response
     */
    public function restore(AgendaRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('biblio::agenda.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/biblio/agenda'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/biblio/agenda/'))
                ->redirect();
        }

    }

}
